@@ -30,6 +30,8 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include <sys/stat.h>
+#include <inttypes.h>
+#include <arpa/inet.h>
 
 #include "ipvswrapper.h"
 #include "global_data.h"
@@ -86,7 +88,7 @@ ipvs_start(void)
 	log_message(LOG_DEBUG, "%snitializing ipvs", reload ? "Rei" : "I");
 	/* Initialize IPVS module */
 	if (ipvs_init()) {
-		if (keepalived_modprobe("ip_vs") || ipvs_init()) {
+		if (!keepalived_modprobe("ip_vs") || ipvs_init()) {
 			log_message(LOG_INFO, "IPVS: Can't initialize ipvs: %s",
 			       ipvs_strerror(errno));
 			no_ipvs = true;
@@ -120,6 +122,47 @@ ipvs_set_timeouts(const ipvs_timeout_t *timeouts)
 		log_message(LOG_INFO, "Failed to set ipvs timeouts");
 }
 
+static size_t
+format_srule(char *buf, const ipvs_service_t *srule)
+{
+	char *bufp = buf;
+
+	if (srule->user.fwmark)
+		return sprintf(buf, "Fwm %" PRIu32 "%s", srule->user.fwmark, srule->af == AF_INET6 ? " inet6" : "");
+
+	inet_ntop(srule->af, srule->af == AF_INET ? (const void *)&srule->nf_addr.ip : (const void *)&srule->nf_addr.in6, bufp, INET6_ADDRSTRLEN);
+	bufp += strlen(bufp);
+	*bufp++ = ':';
+	if (srule->user.protocol == IPPROTO_TCP)
+		strcpy(bufp, "tcp");
+	else if (srule->user.protocol == IPPROTO_UDP)
+		strcpy(bufp, "udp");
+	else if (srule->user.protocol == IPPROTO_SCTP)
+		strcpy(bufp, "sctp");
+	else
+		sprintf(bufp, "%d", srule->user.protocol);
+	bufp += strlen(bufp);
+	bufp += sprintf(bufp, ":%d", ntohs(srule->user.port));
+
+	return (bufp - buf);
+}
+
+static size_t
+format_drule(char *buf, const ipvs_dest_t *drule)
+{
+	char *bufp = buf;
+
+	*bufp++ = ' ';
+	*bufp++ = '-';
+	*bufp++ = '>';
+	*bufp++ = ' ';
+	inet_ntop(drule->af, drule->af == AF_INET ? (const void *)&drule->nf_addr.ip : (const void *)&drule->nf_addr.in6, bufp, INET6_ADDRSTRLEN);
+	bufp += strlen(bufp);
+	bufp += sprintf(bufp, ":%d", ntohs(drule->user.port));
+
+	return (bufp - buf);
+}
+
 /* Send user rules to IPVS module */
 static int
 ipvs_talk(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, ipvs_daemon_t *daemonrule, bool ignore_error)
@@ -173,13 +216,27 @@ ipvs_talk(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, ipvs_daemon_t *dae
 	if (ignore_error)
 		result = 0;
 	else if (result) {
+		char buf[2 + INET6_ADDRSTRLEN + 6 + 5 + 4 + INET6_ADDRSTRLEN + 1 + 5 + 1 + 1];	/* " (" + IPv6 + ":sctp:" + port + " -> " + IPV6 + ":" + port + ")" */
+
 		if (errno == EEXIST &&
 			(cmd == IP_VS_SO_SET_ADD || cmd == IP_VS_SO_SET_ADDDEST))
 			result = 0;
 		else if (errno == ENOENT &&
 			(cmd == IP_VS_SO_SET_DEL || cmd == IP_VS_SO_SET_DELDEST))
 			result = 0;
-		log_message(LOG_INFO, "IPVS cmd %s(%d) error: %s(%d)", ipvs_cmd_str(cmd), cmd, ipvs_strerror(errno), errno);
+
+		buf[0] = ' ';
+		buf[1] = '(';
+		if (cmd == IP_VS_SO_SET_ADD || cmd == IP_VS_SO_SET_DEL || cmd == IP_VS_SO_SET_EDIT)
+			format_srule(buf + 2, srule);
+		else if (cmd == IP_VS_SO_SET_ADDDEST || cmd == IP_VS_SO_SET_DELDEST || cmd == IP_VS_SO_SET_EDITDEST)
+			format_drule(buf + 2 + format_srule(buf + 2, srule), drule);
+		else
+			buf[0] = '\0';
+		if (buf[0])
+			strcat(buf, ")");
+
+		log_message(LOG_INFO, "IPVS cmd %s(%d) error: %s(%d)%s", ipvs_cmd_str(cmd), cmd, ipvs_strerror(errno), errno, buf);
 	}
 	return result;
 }
