@@ -192,6 +192,7 @@ enum snmp_global_magic {
 	SNMP_LINKBEAT,
 	SNMP_LVSFLUSH,
 	SNMP_LVSFLUSH_ONSTOP,
+	SNMP_V3_CHECKSUM_AS_V2,
 	SNMP_IPVS_64BIT_STATS,
 	SNMP_NET_NAMESPACE,
 	SNMP_DBUS,
@@ -267,6 +268,11 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 		long_ret = global_data->lvs_flush_on_stop == LVS_FLUSH_FULL ? 1 :
 			   global_data->lvs_flush_on_stop == LVS_FLUSH_VS ? 3 : 2;
 		return PTR_CAST(u_char, &long_ret);
+#endif
+#ifdef _WITH_VRRP_
+	case SNMP_V3_CHECKSUM_AS_V2:
+		long_ret = SNMP_TruthValue(global_data->v3_checksum_as_v2);
+		return PTR_CAST(u_char, &long_ret);
 #endif
 	case SNMP_IPVS_64BIT_STATS:
 #ifdef _WITH_LVS_64BIT_STATS_
@@ -388,6 +394,9 @@ static struct variable8 global_vars[] = {
 	/* lvsFlushOnStop */
 	{SNMP_LVSFLUSH_ONSTOP, ASN_INTEGER, RONLY, snmp_scalar, 1, {11}},
 #endif
+#ifdef _WITH_VRRP_
+	{SNMP_V3_CHECKSUM_AS_V2, ASN_INTEGER, RONLY, snmp_scalar, 1, {12}},
+#endif
 };
 
 static int
@@ -496,6 +505,7 @@ snmp_agent_close(bool base_mib)
 	if (base_mib)
 		snmp_unregister_mib(global_oid, OID_LENGTH(global_oid));
 	snmp_shutdown(global_name);
+	shutdown_agent();
 
 	snmp_running = false;
 }
