@@ -31,6 +31,8 @@
 /* local includes */
 #include "list_head.h"
 #include "vector.h"
+#include "vrrp_static_track.h"
+
 
 /* Configuration data root */
 typedef struct _vrrp_data {
@@ -62,20 +64,20 @@ extern void *vrrp_buffer;
 extern size_t vrrp_buffer_len;
 
 /* prototypes */
-extern void alloc_static_track_group(const char *);
+extern static_track_group_t *alloc_static_track_group(const char *);
 extern void alloc_saddress(const vector_t *);
 extern void alloc_sroute(const vector_t *);
 extern void alloc_srule(const vector_t *);
-extern void alloc_vrrp_sync_group(const char *);
-extern void alloc_vrrp(const char *);
+extern vrrp_sgroup_t *alloc_vrrp_sync_group(const char *);
+extern vrrp_t *alloc_vrrp(const char *);
 extern void alloc_vrrp_unicast_peer(const vector_t *);
 extern void alloc_vrrp_track_if(const vector_t *);
-extern void alloc_vrrp_script(const char *);
+extern vrrp_script_t *alloc_vrrp_script(const char *);
 extern void free_vscript(vrrp_script_t *);
 extern void alloc_vrrp_track_script(const vector_t *);
 extern void alloc_vrrp_track_file(const vector_t *);
 #ifdef _WITH_TRACK_PROCESS_
-extern void alloc_vrrp_process(const char *);
+extern vrrp_tracked_process_t *alloc_vrrp_process(const char *);
 extern void free_vprocess(vrrp_tracked_process_t *);
 extern void alloc_vrrp_track_process(const vector_t *);
 #endif
