@@ -29,43 +29,48 @@
 
 #include "logger.h"
 #include "list_head.h"
+#include "global_data.h"
 
 #include "vrrp.h"
 #include "vrrp_data.h"
 #include "vrrp_print.h"
 #include "utils.h"
 
-static const char *dump_file = KA_TMP_DIR "/keepalived.data";
-static const char *stats_file = KA_TMP_DIR "/keepalived.stats";
 
 void
 vrrp_print_data(void)
 {
-	FILE *file = fopen_safe(dump_file, "w");
+	FILE *fp;
 
-	if (!file) {
-		log_message(LOG_INFO, "Can't open %s (%d: %s)",
-			dump_file, errno, strerror(errno));
+	fp = open_dump_file("keepalived.data");
+
+	if (!fp)
 		return;
-	}
 
-	dump_data_vrrp(file);
+	dump_data_vrrp(fp);
 
-	fclose(file);
+	fclose(fp);
 }
 
 void
 vrrp_print_stats(bool clear_stats)
 {
-	FILE *file = fopen_safe(stats_file, "w");
+	FILE *file;
 	vrrp_t *vrrp;
+	const char *stats_file;
+
+	stats_file = make_tmp_filename("keepalived.stats");
+
+	file = fopen_safe(stats_file, "w");
 
 	if (!file) {
 		log_message(LOG_INFO, "Can't open %s (%d: %s)",
-			stats_file, errno, strerror(errno));
+				stats_file, errno, strerror(errno));
+		FREE_CONST(stats_file);
 		return;
 	}
 
+	FREE_CONST(stats_file);
 
 	list_for_each_entry(vrrp, &vrrp_data->vrrp, e_list) {
 		fprintf(file, "VRRP Instance: %s\n", vrrp->iname);
